%function [pf0_mrc,pd0_mrc] = roc_thvar_mrc_th(K,M,N,mean_shadow,sigma_shadow,pfk,pdk)
% [pf0_mrc,pd0_mrc] = roc_thvar_mrc_th(K,D,pfk,pdk)
% 
% Compute large N theoretical ROC for MRC (One shot (Shadowing + Path Loss) + Est. CSI) 
%
% -- In:
% K is the number of sensors
% D is the matrix taking into account both path loss and shadowing effects
% pfk is the sensor false-alarm probability (i.i.d.)
% pdk is the sensor detection probability (i.i.d.)
% -- Out:
% pf0_mrc is the vector of system false alarm probabilities vs gamma
% pd0_mrc is the vector of system false alarm probabilities vs gamma

%% Sim Parameters Init
clear 
K = 6;
M = 128;
N = 32;
mean_shadow = 2;
sigma_shadow = 4;
pfk = 0.01;
pdk = 0.5;

% Compute threshold vector for MRC
gamma = linspace(-10,10,1000);

% Init Pd0 and Pf0 vectors
pd0_mrc_tr = [];
pf0_mrc_tr = [];

%% Generate matrix D (See paper Ngo-Marzetta IEEE TC 2013)
 
 r_h = 100; % meters (reference distance for power attenuation)
 r_max = 1000; % meters (max sensors-DFC distance)
 ni = 3; % path loss exponent
 
 % Conversion to natural logaritm
 coeff = (10/log(10)); 
 m_s = mean_shadow / coeff;
 dev_s = sigma_shadow / coeff;
 
 % Generate powers
 pk = lognrnd(m_s,dev_s,K,1);
 
 % Generate uniformly distributed sensors around r_h < DFC < r_max
 phi = (2*pi) * rand(K,1);
 r_t = (1-((r_h/r_max)^2)) * rand(K,1) + ((r_h/r_max)^2);
 
 % Obtain cartesian coordinates of random positions
 x = r_max * sqrt(r_t) .* cos(phi);
 y = r_max * sqrt(r_t) .* sin(phi);
 
 % Get sensors-DFC distance vector
 r = sqrt(x.^2 + y.^2);
 
 % Get power attenuations vector
 beta = pk ./ ((r/r_h).^ ni);
 
 k = 0:1:K-1;
 den = sum(exp((-0.1)*k));
 num = exp((-0.1)*k);
 lambda = num./den;
 den_h = sum(exp((-0.1)*(K-k)));
 num_h = exp((-0.1)*(K-k));
 lambda_h = num_h./den_h;
 %lambda_h = fliplr(lambda);
 
 avg_del = (sum(k.*lambda)).^2; %average delay
  
 for q = 2:1:M
 d(q) = mod(q-1,M);
 c(q) = sum(lambda.*exp(-1i*2*pi*k*d(q)));
 end
 C = abs((sum(c))^2);
 
 D = diag(beta);
 
 rho_bar = sqrt(1/N);
 %rho = rho_bar * sqrt(1/N);  % Tx energy (reporting phase)

%% Get auxiliary parameters for theoretical performance computation

% Get all combinations of transmitted vectors
bit_all = de2bi(0:2^K-1,'left-msb');
x_all = 2 * bit_all' - 1;

%rho_bar = 1; % is the proportional constant for reporting phase
%rho_p_bar = 1; % is the proportional constant for training phase
%tau = 100; % minimum length of training sequence required for meaningful MMSE estimation
sigma2 = 1; % the noise power is set as unit
beta = diag(D)'; % powers vector

% P(x|H_1) vector
pv_1 = prod( pdk * (bit_all') + (1-pdk) * not(bit_all'), 1);

% P(x|H_0) vector
pv_0 = prod( pfk * (bit_all') + (1-pfk) * not(bit_all'), 1);

% Obtain mean and variance vector of Lambda_mrc|G_hat,x (circle)
mean_v_mrc_tr = ((M.*sqrt(2 * N.* rho_bar)) / (sqrt(sigma2+avg_del+C))) .* (((((beta).*(sqrt(lambda)).*(sqrt(lambda_h)))*x_all)) / (sqrt(sum((beta.*lambda_h))))); % (row) vector of means
sigma2mrc = 1;

%% Tx vector possible combinations Loop
for g = gamma

    % display current threshold
    g

    % Get standard normal variable    
    nrv = (- mean_v_mrc_tr + g)/sqrt(sigma2mrc);

    % Obtain Pd0 and Pf0 as a sum of Q-functions
    Pd0_t = sum (qfunc(nrv).* pv_1);
    Pf0_t = sum (qfunc(nrv).* pv_0);

    % Append obtained values to the corresponding vectors
    pf0_mrc_tr = [pf0_mrc_tr;Pf0_t];
    pd0_mrc_tr = [pd0_mrc_tr;Pd0_t];

end

%% Save results
% save(['ROC- th. large N - MRC(est. CSI) -- K ',num2str(K),'- Pfk ',num2str(pfk),'- Pdk ',num2str(pdk),'.mat']);
%end

%figure;
semilogx(pf0_mrc_tr, pd0_mrc_tr);
axis([10^(-3) 1 0.1 1])
hold on
